/**
  * Created by ariellemoro on 16.11.16.
  */
import collection.mutable.{ListBuffer, Map}

object DijkstraAlgo {

  def main(args: Array[String]): Unit = {
    //Set main variables
    val graph = Map("s" -> Map("u" -> 10, "x" -> 5), "u" -> Map("v" -> 1, "x" -> 2), "v" -> Map("y" -> 4), "x" -> Map("v" -> 9, "y" -> 2), "y" -> Map("u" -> 5))
    val source = "s"
    val destination = "y"
    //Compute the shortest paths from graph and source
    var previousNode = computeShortestPaths(graph, source)
    //Extract the shortest path from the previous node list
    extractShortestPath(previousNode, destination)
  }

  def computeShortestPaths(graph: Map[String, Map[String, Int]], source: String): Map[String, String] = {
    var distances = Map.empty[String, Int]
    var previousNode = Map.empty[String, String]
    var unvisitedNodes = new ListBuffer[String]
    for(node <- graph.keys){
      distances(node) = Int.MaxValue
      previousNode(node) = ""
      unvisitedNodes += node
    }
    distances(source) = 0
    //Compute the shortest paths
    while(unvisitedNodes.nonEmpty){
      //Find minimum node distance in "distances" list
      var distanceMin = Int.MaxValue
      var bestMinNode = source
      for (node <- distances.keys) {
        if (distances(node) < distanceMin && unvisitedNodes.contains(node)) {
          distanceMin = distances(node)
          bestMinNode = node
        }
      }
      var currentNode = bestMinNode
      unvisitedNodes -= currentNode
      //Explore neighbour nodes of the currentNode
      var neighbours = graph(currentNode)
      for ((neighbourNode, neighbourDist) <- neighbours) {
        if (unvisitedNodes.contains(neighbourNode)) {
          if ((distanceMin + neighbourDist) < distances(neighbourNode)) {
            distances(neighbourNode) = distanceMin + neighbourDist
            previousNode(neighbourNode) = currentNode
          }
        }
      }
    }
    return previousNode
  }

  def extractShortestPath(previous: Map[String, String], destination: String): Unit ={
    var currentNode = destination
    var shortestPath = new ListBuffer[String]
    for(node <- previous.keys){
      if(node == currentNode) {
        shortestPath += node
        currentNode = previous(currentNode)
      }
    }
    shortestPath += currentNode
    shortestPath = shortestPath.reverse
    //Print the shortest path
    for(node <- shortestPath){
      println(node)
    }
  }

}
